// Scripts used to test SAS rule code
// @TODO: I18N



// Testing flag should be false in production
var rapTest = false;

// List of tabs that share SAS testing controls
var sasTabs = new Array("sasEditor", "sasLog", "testResults");

// "SAS operation in progress" graphic says a SAS operation is in progress
var sasSpinner = "common/images/spinner.gif";


// Adds a listener for the screen to catch the indicated type of mouse event
function addListener(type, callback)
{
  if (document.addEventListener)
    document.addEventListener(type, callback, false);
  else if (document.attachEvent)
    document.attachEvent("on"+type, callback, false);
}



// Removes a named listener of the indicated type for the screen
function removeListener(type, callback)
{
  if (document.removeEventListener)
    document.removeEventListener(type, callback, false);
  else if (document.detachEvent)
    document.detachEvent("on"+type, callback, false);
}




// Return actual position of an element on the screen
function getLeftTop(obj) {
  if (!obj)
     return null;

  var curleft = curtop = 0;
  if (obj.offsetParent) {
    do {
      curleft += obj.offsetLeft;
      curtop += obj.offsetTop;
    } while (obj = obj.offsetParent);
    return [curleft,curtop];
  }
}


// Creates a browser-independent event object
function CommonEvent(e)
{
  if (e) {
    this.e = e;
  } else {
    this.e = window.event;
  }
  
  if (e.keyCode)
    this.keyCode = e.keyCode;
  else if (e.which)
    this.keyCode = e.which;
  e.character = String.fromCharCode(e.keyCode);
  
  if (e.pageX) {
    this.x = e.pageX;
  } else {
    this.x = e.clientX;
  }

  if (e.pageY) {
    this.y = e.pageY;
  } else {
    this.y = e.clientY;
  }

  if (e.target) {
    this.target = e.target;
  } else {
    this.target = e.srcElement;
  }
}  



// Enable 'Save' and hide previous saveStatus message when something changes for new and Coding rules
enableSaveButton.buttonEnabled = true;
function enableSaveButton(e)
{
  var isNewOrCoding = (!document.ruleForm.state || document.ruleForm.state.value == "Coding");
  var isEnableableEvent = (e) ? (new CommonEvent(e)).target.id.indexOf('doNotDisable') < 0 : true;
  if (isNewOrCoding  &&  isEnableableEvent)
  {
    enableSaveButton.buttonEnabled = true;
    bar.setEnabled( 0, true );

    var saveStatusArea = document.getElementById('saveStatusDisplay');
    if (saveStatusArea)
      saveStatusArea.innerHTML = '';
  }
}

// Disable 'Save' when save complete
function disableSaveButton(theWindow)
{
  theWindow = (theWindow) ? theWindow : window;
  theWindow.enableSaveButton.buttonEnabled = false;
  theWindow.bar.setEnabled( 0, false );
}




// Reflects the results of a cancel, status, or reset request in the rule detail screen
function showStatusCancelComplete()
{
  var resultDoc = window.frames.statusCancelResultsFrame.document;
  var rc = resultDoc.getElementById('rc').innerHTML;
  var pid = resultDoc.getElementById('pid').innerHTML;

  if (resultDoc.getElementById('action').innerHTML == 'cancelProcess')
  {
    var msg;

    if (rc == 0)
    {
      msg = "<b><font color='blue'>Canceled!</font></b>&nbsp;&nbsp;&nbsp;";
      setErrorWarningDisplay(document, 'sasLog', 'SAS Log', msg)
    }
    else if (rc == 4)
    {
      msg = "No active process";
      //alert(msg);
      setErrorWarningDisplay(document, 'sasLog', 'SAS Log', msg)
    }
    else if (rc == 8)
    {
      msg = "<b><font color='green'>Canceled (with warnings)</font></b>&nbsp;&nbsp;&nbsp;";
      setErrorWarningDisplay(document, 'sasLog', 'SAS Log', msg)
    }
    else 
    {
      msg = "Error encountered canceling pid " + pid + " (rc=" + rc + ").  If this error recurs, please contact a system adminstrator.";
      alert(msg);
    }
  }


  else if (resultDoc.getElementById('action').innerHTML == 'resetConnection')
  {
    var msg;
    msg = "<b><font color='blue'>Reset!</font></b>&nbsp;&nbsp;&nbsp;";
    setErrorWarningDisplay(document, 'sasLog', 'SAS Log', msg)
    msg = "The connection to the SAS workspace server has been closed.  Future requests to run SAS code will create a new connection.";
    alert(msg);
  }


  else // request == 'requestStatus'
  {
    var msg = (pid != '.' && pid != '') ? "Process ID: "+pid+" / " : "";

    if (rc == 0)
      msg += resultDoc.getElementById('processStatus').innerHTML;
    else if (rc == 4)
      msg = "No active process";
    else if (rc == 8)
      msg += resultDoc.getElementById('processStatus').innerHTML + " (warnings)";
    else 
      msg += resultDoc.getElementById('processStatus').innerHTML + "\n" +
             "Error encountered querying the status of pid " + pid + " (rc=" + rc + ").  If this error recurs, please contact a system adminstrator.";

    alert(msg);
  }
}


// Copies results of a Save operation to the containing page
function showSaveComplete()
{
  // Adjust datetimes to local from GMT
  adjustDatetimes();

  copyDivContents( 'saveStatusDisplay' );

  // TODO: Detect this with RC, not text compare with 'error'
  if (document.getElementById('saveStatusDisplay').innerHTML.toLowerCase().indexOf("error") < 0)
  {
    // Copy returned display values to containing page
    copyDivContents( 'createdInfo' );
    copyDivContents( 'updatedInfo' );
    // copyDivContents( 'checkoutInfo' );
    copyDivContents( 'baseRuleIdInfo' );
    copyDivContents( 'ruleTypeInfo' );
  
    // Fill in updated form values, too
    copyFormValue('resultsForm', 'ruleForm', 'ruleId');
    copyFormValue('resultsForm', 'ruleForm', 'lockVersion');
    copyFormValue('resultsForm', 'ruleForm', 'version');
    copyFormValue('resultsForm', 'ruleForm', 'baseRuleId');
    copyFormValue('resultsForm', 'ruleForm', 'state');
    copyFormValue('resultsForm', 'ruleForm', 'checkoutUser');
    copyFormValue('resultsForm', 'ruleForm', 'slotId');
    copyFormValue('resultsForm', 'ruleForm', 'ruleTypeId'); 
    copyFormValue('resultsForm', 'ruleForm', 'executionOrderType');
    copyFormValue('resultsForm', 'ruleForm', 'executionOrderPriority');
    copyFormValue('resultsForm', 'ruleForm', 'multiOrgId');
   
    // Update title of containing page
    var rt = '';
    if (parent.window.document.getElementById('isTenantInstall') && parent.window.document.getElementById('isTenantInstall').value == 'false')
        rt = " (" + parent.window.document.getElementById('ruleTypeInfo').innerHTML.replace(/^\s*|\s*$/g,"") +  ")";
    parent.window.document.title = "Rule " + parent.window.document.ruleForm.baseRuleId.value +
      ":" + rt + " " + parent.window.document.ruleForm.name.value; 
  
    // Refresh user variable usage by rules
    for (var f in FieldLabelMap)
    {
      if (FieldLabelMap[f] instanceof Function) continue;
      if (FieldLabelMap[f].substr(0,4) == "ruv_"  &&  parent.window.FieldLabelMap[f] != FieldLabelMap[f])
        parent.window.FieldLabelMap[f] = FieldLabelMap[f];
    }
    parent.window.loadFieldExpansions();

    // Copy refreshed dependency possibilities
    parent.window.preRequisiteLabelMap = new Array();
    for (var p in preRequisiteLabelMap)
    {
      if (preRequisiteLabelMap[p] instanceof Function) continue;
      parent.window.preRequisiteLabelMap[p] = preRequisiteLabelMap[p];
    }
    parent.window.loadPrereqExpansions();

    // Refresh containing page's opener to pick up freshly-saved data
    refreshOpener(parent.window);

    // Chain operations if requested
    var nextOp = document.getElementById("nextOp").value;
    // Removed the following so SAVE would not clear interesting log and results
    //if (nextOp == "check")
    //   parent.window.runSasCode(false,-1);
    //else
    if (nextOp.substr(0,10) == 'runSasCode')
       parent.window.runSasCode(nextOp.split(" ")[1],nextOp.split(" ")[2],nextOp.split(" ")[3]);
  }

  // Fix-up containing page's display to disable Save until next change
  disableSaveButton(parent.window)
}


// Copies div contents from results page to containing page
function copyDivContents(divId)
{
  var target = parent.window.document.getElementById(divId);
  if (!target)
    alert("No target div in copyDivContents() " + divId);
  var source = document.getElementById(divId);
  if (!source)
    alert("No source div in copyDivContents() " + divId);
  target.innerHTML = source.innerHTML;
}

// Copies form values from results page to containing page, creating the target tag if necessary
function copyFormValue(sourceFormName, targetFormName, tagName)
{
  var target = parent.window.document[ targetFormName ];
  var source = document[ sourceFormName ];
  if (!target[ tagName ])
  {
    var newTag = parent.window.document.createElement('input');
    newTag.type = "hidden";
    newTag.name = tagName;
    newTag.id = tagName;
    target.appendChild(newTag);
  }
  target[tagName].value = source[tagName].value;
}





// Resizes variable-size areas as needed for rules popup screen when window changes size
function rulesTabPopupResizeHandler()
{
  // Get browser-dependent window size and available-area adjustment
  var winH, winW;
  var adjH, adjW;
  if (window.innerHeight)
  {
    winH = parseInt(window.innerHeight);
    adjH = -8;
    winW = parseInt(window.innerWidth);
    adjW = -10;
  }
  else
  { 
    winH = parseInt(document.body.offsetHeight) - 4;
    adjH = +4;
    winW = parseInt(document.body.offsetWidth) - 4;
    adjW = -14;
  }
  // Preserve size
  var theDate = new Date();
  theDate.setDate(theDate.getDate()+1000);
  setCookie("rule_detail_window_height", winH, theDate);
  setCookie("rule_detail_window_width",  winW, theDate);

  // Adjust for actual available inner size
  winH = winH + adjH;
  winW = winW + adjW;

  // Resize areas for new window size
  rulesTabResizer( winH, winW, document.getElementById('customCode'), .82, .98 );
  rulesTabResizer( winH, winW, document.getElementById('canvas') );
  rulesTabResizer( winH, winW, document.getElementById('code'), 1, 1 );
  rulesTabResizer( winH, winW, document.getElementById('gutter'), 1, .02 );
  rulesTabResizer( winH, winW, document.getElementById('logFrame') );
  rulesTabResizer( winH, winW, document.getElementById('resultsFrame') );
  rulesTabResizer( winH, winW, document.getElementById('expansionSelectorDiv'), .95, .40 );
}


function rulesTabResizer(winH, winW, areaTag, heightFactor, widthFactor)
{
  if (!areaTag)
     return;
  heightFactor = (heightFactor) ? heightFactor : 1;
  widthFactor = (widthFactor) ? widthFactor : 1;
  var areaTop;
  try {
    areaTop = getLeftTop(areaTag)[1];
  }
  catch(ex) {
    // alert("Exception accessing getLeftTop(): " + ex);
  }
  if (!areaTop)
    areaTop = parseInt(areaTag.style.top);
  if (!areaTop) {
    return;
  }
  var height = Math.floor( (winH - areaTop - 20) * heightFactor );
  var width = Math.floor( (winW - 26) * widthFactor );
  // alert(areaTag.id + ' h='+height + ' w='+width+ ' top='+areaTop + ' hf='+heightFactor + ' wf='+widthFactor); 
  if (heightFactor != -1)
     areaTag.style.height = ((height > 0) ? height : 100) + 'px';
  if (widthFactor != -1)
     areaTag.style.width = ((width > 0) ? width : 100) + 'px';
}



// Prompts the user to save changes when the rule detail window closes
function beforeUnloadHandler(e)
{
  if (!bypassSaveCheck  &&  enableSaveButton.buttonEnabled)
    return "You will lose all changes made since your last save.";
}


// Sets occurences of a tag in all rule SAS tabs to a value (and saves cookie)
function setTabValue( callerTag, doc, tagName, value )
{
    var callerTagId=' ';
    if (callerTag && callerTag.id)
        callerTagId = callerTag.id;
    var tag = eval("doc.ruleForm."+tagName);
    if (tag && tag.id != callerTagId)
        tag.value = value;

    for (var i=0; i<sasTabs.length; i++)
    {
        tag = eval("doc.ruleForm."+sasTabs[i]+"_"+tagName);
        if (tag && tag.id != callerTagId)
            tag.value = value;
    }
    setCookie("rule_"+tagName,value);
}



// Clears the status (SAS processes and saves
function clearStatus()
{
  var statusArea = top.document.getElementById('errorWarningDisplay');
  if (statusArea) statusArea.innerHTML = '';
  statusArea = top.document.getElementById('saveStatusDisplay');
  if (statusArea) statusArea.innerHTML = '';
}


// Sets the status in the errorWarning display area
function setErrorWarningDisplay( doc, destinationTab, destinationLabel, contents )
{
    var statusArea = doc.getElementById( "errorWarningDisplay" );
    if (statusArea)
    {
        if (destinationTab != "")
          statusArea.innerHTML = "<span onclick='swapGridContents(\"" + destinationTab + "\",1)'>" +  contents + "</span>";
        else
          statusArea.innerHTML = "<span>" + contents + "</span>";
        statusArea.title = "Show " + destinationLabel;
        statusArea.style.cursor = "pointer";
    }
}


// Sets the status in the saveStatus display area
function setSaveStatusDisplay( doc, confirmationText, errorText )
{
    var statusArea = doc.getElementById( "saveStatusDisplay" );
    if (statusArea)
    {
        if (errorText)
           statusArea.innerHTML = "<span onclick='alert(\"" + errorText + "\")'>" + confirmationText + "</span>";
        else
           statusArea.innerHTML = confirmationText ;
        statusArea.title = errorText;
        statusArea.style.cursor = "pointer";
    }
}


// Returns a value of a tag without error even if the tag is null
function getTagValue( tag )
{
  return (tag) ? tag.value : "";
}

// Returns innerHTML of a tag without error even if the tag is null
function getTagHTML( tag )
{
  return (tag) ? tag.innerHTML : "";
}



// Returns an indicator of whether a SAS submission is underway
function isSasActive( action )
{
    if (rapTest) alert('isSasActive'+action)

    // Find status area in this frame or in its parent
    var isAct = false;
    var statusArea = document.getElementById("errorWarningDisplay");
    if (!statusArea)
        statusArea = parent.document.getElementById("errorWarningDisplay");

    // If the spinner is not in the status area, show that we are not busy
    if (!statusArea || statusArea.innerHTML.indexOf(sasSpinner) < 0)
        isAct = false ;
    else if (action == 'complain')
    {
        alert('A SAS submission is in progress.  Please wait.');
        isAct = true;
    } else if (action == 'ask')
        isAct = (! confirm('A SAS submission is in progress.  Continue anyway?'))
    else
        isAct = true;

    if (rapTest) alert('exitisSasActive'+isAct);
    return isAct;
}



// Turns used and changed variable names into surrogate keys
function convertVariableNamesToIds()
{
  if (document.ruleForm.usedVariableIds.value == ".")
      document.ruleForm.usedVariableIds.value = _convertVariableNamesToIds(document.ruleForm.usedVariables.value);
  if (document.ruleForm.changedVariableIds.value == ".")
      document.ruleForm.changedVariableIds.value = _convertVariableNamesToIds(document.ruleForm.changedVariables.value);
}

// Turns variable names into surrogate keys
function _convertVariableNamesToIds(vnames)
{
  var varnames = " " + vnames.toLowerCase() + " ";
  var varids = "";
  for (var f in FieldLabelMap)
  {
    if (FieldLabelMap[f] instanceof Function) continue;
    if (varnames.indexOf(" " + FieldLabelMap[f].split(":")[0] + " ") >= 0)
      varids += f + " ";
  }
  return varids.replace(/^\s*|\s*$/g,"");  // Trim whitespace
}

// Combines used and changed variable id lists into a list of variable names with * for changed vars
function convertVariableIdsToDisplayable(usedVariables, changedVariables)
{
  var usedVars = _convertVariableIdsToNames(usedVariables).toUpperCase().split(" ");
  var changedVars = _convertVariableIdsToNames(changedVariables).toUpperCase().split(" ");
  var vars = new Array();
  var i = 0;
  for (i=0; i<usedVars.length; i++)
    vars[usedVars[i]] = '';
  for (i=0; i<changedVars.length; i++)
    vars[changedVars[i]] = "*";
  var display = new Array();
  i = 0;
  for (var v in vars)
  {
    if (v == '' || vars[v] instanceof Function)
      continue;
    display[i++] = v + vars[v];        
  }
  return display.join(", ");
}

// Turns a list of variable ids into a list of variable names
function _convertVariableIdsToNames(ids)
{
  var varids = ids.replace(/^\s*|\s*$/g,"");
  if (varids=="") return "";

  varids = varids.split(" ");
  var varnames = new Array();
  var ii = 0;
  for (var i=0; i<varids.length; i++)
  {
    if (FieldLabelMap[varids[i]])
       varnames[ii++] = FieldLabelMap[varids[i]].split(":")[0].toUpperCase();
  }
  varnames.sort();
  return varnames.join(" ");
}


// Turns component selections for this rule into an array of ids (includes implicit component ids)
function getComponentIdArray() {
  var compIdsSet = new Array();
  // Get the components from checkboxes
  var inputTags = document.getElementsByTagName("INPUT");
  for (var t=0; t<inputTags.length; t++) {
    var inputTag=inputTags[t];
    if (inputTag.id && inputTag.id.substr(0,10)=="component_" && inputTag.checked) {
      var assocIds = inputTag.value.split(":")[1].split(" ");
      for (var a=0; a<assocIds.length; a++)
        compIdsSet[assocIds[a]] = assocIds[a];
    }
  }
  var compIdsArray = new Array();
  for (var c in compIdsSet) 
    compIdsArray.push(c);
  for (var s=0; s<SystemComponentIds.length; s++)
    compIdsArray.push(SystemComponentIds[s]);
  compIdsArray.sort();
  return compIdsArray;
}

// Turns preRequisite handles into keys
function convertPreRequisiteHandlesToIds()
{
  document.ruleForm.preRequisiteIds.value = _convertPreRequisiteHandlesToIds(document.ruleForm.preRequisites.value);
}

// Turns preRequisites into keys (easy at the moment because they are just #nn, but may soon be ruleNames)
function _convertPreRequisiteHandlesToIds(reqs)
{
  return reqs.replace(/#/g,"").replace(/^\s*|\s*$/g,""); 
}

// Turns a list of prerequisite IDs into labels for display
function convertPreRequisiteIdsToDisplayable(ids)
{
  if (ids=="") return "";
  var reqIds = ids.replace(/^\s*|\s*$/g,""); // Strip whitespace
  reqIds =  reqIds.split(" ");
  var reqLabels = new Array();
  var ii = 0;
  for (var i=0; i<reqIds.length; i++)
  {
    if (preRequisiteLabelMap[reqIds[i]])
       reqLabels[ii++] = escapeXml(preRequisiteLabelMap[reqIds[i]].replace(/\s{2,}/g," ")) + '&nbsp;&nbsp;<i>(for %HasRuleFired)</i>'; // compress multiple blanks S0881932 */
    else
       reqLabels[ii++] = reqIds[i];
  }
  reqLabels.sort(prereqComparator);
  return reqLabels.join("<BR>");
}

// Returns -1,0,+1 for sorting rule labels by ruleType, ruleName,ruleNum
function prereqComparator( lab1, lab2 )
{
    var parts1 = lab1.split(" ");
    var parts2 = lab2.split(" ");
    var len1 = parts1.length;
    var len2 = parts2.length

    var ruleType1 = len1>1 ? parts1[1] : '';
    var ruleType2 = len2>1 ? parts2[1] : '';
    if (ruleType1 < ruleType2)  return -1
    if (ruleType1 > ruleType2)  return  1;

    var ruleName1 = len1>2 ? lab1.substr( parts1[0].length + parts1[1].length + 2 ).toLowerCase() : '';
    var ruleName2 = len2>2 ? lab2.substr( parts2[0].length + parts2[1].length + 2 ).toLowerCase() : '';
    if (ruleName1 < ruleName2)  return -1;
    if (ruleName1 > ruleName2)  return  1;

    var ruleNumb1 = len1>0 ? parts1[0] : '';
    var ruleNumb2 = len2>0 ? parts2[0] : '';
    if (ruleNumb1 < ruleNumb2)  return -1;
    if (ruleNumb1 > ruleNumb2)  return  1;

    return 0;
}


// Possibly save rule prior to SAS submission
function confirmSavePriorToSasSubmit(debugFlag, obsCount, codeGenOnly)
{
  document.ruleForm.codeGenOnly.value = codeGenOnly=='true' ? 'true' : '';
  if (enableSaveButton.buttonEnabled)
  {
    if (confirm("You must save the rule before running this request.\n\nClick OK to save the rule or Cancel to cancel the request."))
      submitForm('runSasCode ' + debugFlag + ' ' + obsCount + ' ' + codeGenOnly);
    return;
  }
  runSasCode(debugFlag, obsCount, codeGenOnly);
}


// Always save rule prior to SAS submission
function runSasCodeAfterSave(debugFlag, obsCount, codeGenOnly)
{
  document.ruleForm.codeGenOnly.value = codeGenOnly=='true' ? 'true' : '';
  submitForm('runSasCode ' + debugFlag + ' ' + obsCount + ' ' + codeGenOnly);
}


// Runs SAS code from any tab if no SAS operation already in progress
function runSasCode(debugFlag, obsCountOrig, codeGenOnly)
{
    document.ruleForm.codeGenOnly.value = codeGenOnly=='true' ? 'true' : '';
    var obsCount = parseInt("" + obsCountOrig);
    if (!obsCount || obsCount<-1)
    {
        alert("Limit row count '" + obsCountOrig + "' is not a positive integer");
        return;
    }

    if (!isSasActive('complain') && validateForm())
    {
        // Display the spinner while we wait
        setErrorWarningDisplay( document, 'sasLog', 'SAS Log', '<image title="SAS submission in progress..." src="' + sasSpinner + '" border="0" height="15px" width="15px"/>' );

        // Clear the gutter
        var gutter = document.getElementById("gutter");
        if (gutter)
            gutter.innerHTML = "&nbsp;";

        // Copy sasCode and other rule info from rule form fields to SAS tester form fields
        var logForm = logFrame.document.logForm;
        logForm.sasCode.value = getTagValue( document.ruleForm.code );
        logForm.baseRuleId.value = getTagValue( document.ruleForm.baseRuleId );
        logForm.ruleId.value = getTagValue( document.ruleForm.ruleId );
        logForm.ruleTypeId.value = getTagValue( document.ruleForm.ruleTypeId );
        logForm.transactionId.value = getTagValue( document.ruleForm.transactionId );
        logForm.queueId.value = getTagValue( document.ruleForm.queueId );
        logForm.alertType.value = getTagValue( document.ruleForm.alertType );
        logForm.version.value = getTagValue( document.ruleForm.version );
        logForm.state.value = getTagValue( document.ruleForm.state );
        logForm.checkoutUser.value = getTagValue( document.ruleForm.checkoutUser );
        logForm.slotId.value = getTagValue( document.ruleForm.slotId );
        logForm.multiOrgId.value = getTagValue( document.ruleForm.multiOrgId );
        document.getElementById("componentIds").value = getComponentIdArray().join(" ");
        logForm.componentIds.value = getTagValue( document.ruleForm.componentIds );
        logForm.executionOrderType.value = getTagValue( document.ruleForm.executionOrderType );
        logForm.executionOrderPriority.value = getTagValue( document.ruleForm.executionOrderPriority );
        logForm.earlyDate.value = getTagValue( document.ruleForm.earlyDate );
        logForm.lateDate.value = getTagValue( document.ruleForm.lateDate );
        logForm.account.value = getTagValue( document.ruleForm.account );
        logForm.samplePercent.value = getTagValue( document.ruleForm.samplePercent );
        logForm.sampleKey.value = getTagValue( document.ruleForm.sampleKey );
        logForm.codeGenOnly.value = getTagValue( document.ruleForm.codeGenOnly );
        logForm.generatedCode.value = '';
        // Do not allow selection of data to process from webapp right now (06Jul07,dmf)
        // logForm.usingArchiveData.checked = (document.ruleForm.usingArchiveData && document.ruleForm.usingArchiveData.checked);
        logForm.estimateClassvars.value = getTagValue( document.ruleForm.estimateClassvars );

        // Remember subsetting values
        setCookie("rule_earlyDate", getTagValue( document.ruleForm.earlyDate));
        setCookie("rule_lateDate", getTagValue( document.ruleForm.lateDate));
        setCookie("rule_account", getTagValue( document.ruleForm.account));
        setCookie("rule_samplePercent", getTagValue( document.ruleForm.samplePercent));
        setCookie("rule_sampleKey", getTagValue( document.ruleForm.sampleKey));
    
        // Copy passed debugFlag and obsCount for SAS tester form fields
        if (!debugFlag || debugFlag=='false') // S0747537
            logForm.debugFlag.value = 0;
        else
            logForm.debugFlag.value = 1;
        logForm.obsCount.value = (obsCount!=null) ? obsCount : document.ruleForm.obsCount.value;

        // Clear out results fields and SAS log to avoid confusion
        document.ruleForm.errorCount.value = '';
        document.ruleForm.warningCount.value = '';
        document.getElementById('generatedCode').value = 'Loading...';
        document.ruleForm.usedVariables.value = '';
        document.ruleForm.usedVariableIds.value = '.';
        document.ruleForm.changedVariables.value = '';
        document.ruleForm.changedVariableIds.value = '.';
        document.ruleForm.preRequisites.value = '';
        document.ruleForm.preRequisiteIds.value = '.';
        logFrame.document.getElementById("actualSasLog").style.display = "none";
        resultsFrame.document.getElementById("actualResults").style.display = "none";        
        resultsFrame.document.getElementById("resultsRefresher").style.display = "none";
        var tcnt = resultsFrame.document.getElementById("totalTransactionCount");
        if (tcnt) tcnt.value = "";

        // If not IE, swap to the SAS log tab briefly before submitting
        if (activeTab[0] != 'sasLog' && navigator.appName.toLowerCase().indexOf("microsoft") < 0)
        {
            var restoreTab = activeTab[0];
            swapGridContents('sasLog',1);
            setTimeout( "submitSasLogForm('"+restoreTab+"');",0 );
        }
        else
            submitSasLogForm('');
    }
    document.ruleForm.codeGenOnly.value='';
}

// Submits the form to run SAS code and restores the submitters tab (if necessary)
function submitSasLogForm(restoreTab) {
    // Submit SAS code and restore original tab
    logFrame.document.logForm.submit();
    if (restoreTab != '')
        eval( 'swapGridContents("' + restoreTab + '",1);' );
}



// Enables menu buttons once page has loaded and suppresses n/a buttons
function enableButtons()
{

  if (window.actionsMenu && document.ruleForm)
  {
    var stateTag = document.ruleForm.state;
    if (!stateTag || stateTag.value == "Coding")
    {
      bar.setEnabled( 1, true );
      bar.setEnabled( 2, true );
      bar.setEnabled( 3, true );
      bar.setEnabled( 4, true );
      actionsMenu.setEnabled( 0, true );
      actionsMenu.setEnabled( 1, true );
      actionsMenu.setEnabled( 2, true );
      actionsMenu.setEnabled( 3, true );
    }
    else
    {
      document.getElementById('sasLogTab').style.display = "none";
      document.getElementById('testResultsTab').style.display = "none";
    }

    // Always enable printer action
    actionsMenu.setEnabled( printerActionIndex, true );
  }
}


// Return a string of blank-separated values as an associative array 
function getSet(str) {
  var ary = str.split(" ");
  var set = new Array();
  for (var i=0; i<ary.length; i++)
    set[""+ary[i]] = "" + ary[i];
  return set;
}


// Returns the index of the indicated Option value
function findOption(options, value, startFrom) {
  if (!startFrom)
    startFrom = 0;
  for (var i=startFrom; i < options.length; i++) {
    if (options[i].value == value)
       return i;
  }
  return -1;
}


// Reorders component pre-set selections to put most recently used at the top
function reorderPreSetSelections() {
  var selector = document.getElementById("componentPreSetSelector");
  var selectedIndex = selector.selectedIndex;
  var selectedValue = selector.options[selectedIndex].value;
  var selectedText  = selector.options[selectedIndex].text;

  // Load saved presets (or use top few)
  var savedPresets = getCookie("rule_component_presets");
  if (savedPresets) {
    savedPresets = savedPresets.split(" ");
    if (savedPresets.length != SAVED_PRE_SET_COUNT)
      savedPresets = null;
  }
  if (!savedPresets) {
    savedPresets = new Array(); 
    for(var p=SAVED_PRE_SET_COUNT+1; p < SAVED_PRE_SET_COUNT*2+1; p++) savedPresets.push(p);
  }
  else
  //alert('savedPresets=' + savedPresets.join(" ") + " selectedIndex=" + selectedIndex);

  // If a selection other than the top one was made, move things around
  if (selectedIndex > 1)
  {
    // If the selection is not a pre-set, assure it is in the pre-sets
    if (selectedIndex > SAVED_PRE_SET_COUNT)
    {
      var preIx = index(savedPresets,selectedIndex);
      if (preIx >= 0)
        selectedIndex = preIx+1;
      else {
        for (var i=SAVED_PRE_SET_COUNT; i > 1; i--) {
          selector.options[i].value = selector.options[i-1].value;
          selector.options[i].text  = selector.options[i-1].text;
        }
        savedPresets.length = SAVED_PRE_SET_COUNT-1;
        savedPresets.unshift(selectedIndex);
      }
    }
    // If the selection one of the pre-sets was selected, move pre-sets around
    if (selectedIndex <= SAVED_PRE_SET_COUNT) {
      for (var i=selectedIndex; i > 1; i--) {
        selector.options[i].value = selector.options[i-1].value;
        selector.options[i].text  = selector.options[i-1].text;
      }
      savedPresets.splice(selectedIndex-1, 1);
      savedPresets.unshift(findOption(selector.options, selectedValue, SAVED_PRE_SET_COUNT+1));
    }
    selector.options[1].value = selectedValue;
    selector.options[1].text  = selectedText;
    selector.selectedIndex = 1;
  }

  // Populate the pre-set selections
  for (var i=0; i<SAVED_PRE_SET_COUNT; i++) {
    selector.options[i+1].value = selector.options[savedPresets[i]].value;
    selector.options[i+1].text  = selector.options[savedPresets[i]].text;
  }

  // Save the new savedPresets
  var longTime = new Date();
  longTime.setDate(longTime.getDate()+1000);
  //alert('storedPresets=' + savedPresets);
  setCookie("rule_component_presets", savedPresets.join(" "), longTime);
}


// Used to sort an Array of Options by their label
function UNUSED_sortOptionsByText(o1, o2) {
  if (o1.text < o2.text) return -1;
  else
  if (o1.text == o2.text) return 0;
  else
  return 1;
}

// Return position of needle in haystack Array (or -1 if not found)
function index(haystack, needle) {
  for (var i=0; i < haystack.length; i++) {
    if (haystack[i]==needle)
      return i;
  }
  return -1;
}

// Set component selections and pre-set based on contents of componentIds list
function loadComponentSelections() {
  // Set checkboxes for the indicated component IDs
  var selectedCompIdsSet = getSet(document.getElementById("componentIds").value);
  setComponentSelections(selectedCompIdsSet, true);

  // See if any pre-set matches the selected components
  var selectedCompIds = getSet(getComponentIdArray().join(" ")).toString();
  var compPreSetSelector = document.getElementById("componentPreSetSelector");
  var tranCompOptions = compPreSetSelector.options;
  for (var t=0; t < tranCompOptions.length; t++) {
    var tranCompSet = TransactionComponentsMap[tranCompOptions[t].value];
    // if (tranCompSet) alert("'" + tranCompSet.toString() + "'\n'" + selectedCompIds +"'");
    if (tranCompSet && tranCompSet.toString()==selectedCompIds) {
      compPreSetSelector.selectedIndex = t;
      break;
    }
  }
}

// Populates rule SAS test values and other initialization for rules detail onloadHandler
function buildSelectors() {
  if (rapTest) alert("buildSelectors");

  // Should never happen
  if (! document.ruleForm) return;
  
  // Build selections for SAS editor
  reorderPreSetSelections();
  loadComponentSelections();
  loadFieldExpansions(); // Must precede loadOverviewExpansions()
  loadMacroExpansions();
  loadOverviewExpansions();
  loadPrereqExpansions();
  setComponentTitles();  
  
  // Init generated code area to avoid user confusion
  if (document.getElementById("generatedCode").value == "")
      document.getElementById("generatedCode").value = "The generated code is not available until you either request that it be generated or run the rule's SAS code.";
  
  // Populate subsetting values from cookies
  var obsCount = getCookie("rule_obsCount");
  setTabValue(null, document, "obsCount", (obsCount ? obsCount : '10') );
  var earlyDate = getCookie("rule_earlyDate");
  setTabValue(null, document, 'earlyDate', (earlyDate ? earlyDate : 'Earliest'))
  var lateDate = getCookie("rule_lateDate");
  setTabValue(null, document, 'lateDate', (lateDate ? lateDate : 'Latest'))
  var account = getCookie("rule_account");
  setTabValue(null, document, 'account', (account ? account : ''))
  var samplePercent = getCookie("rule_samplePercent");
  setTabValue(null, document, 'samplePercent', (samplePercent ? samplePercent : '1'))
  var sampleKey = getCookie("rule_sampleKey");
  setTabValue(null, document, 'sampleKey', (sampleKey ? sampleKey : ' '))
  
  // Disable 'Run' and 'Debug' buttons if not Coding, Testing, Production, or new rule
  if (document.ruleForm.state && "Coding Testing Production".indexOf(document.ruleForm.state.value) < 0)
  {
    bar.setEnabled( 1, false );
    bar.setEnabled( 2, false );
  }
  
  // Refresh opener if there is a confirmation message present
  if (document.getElementById('refreshOpenerIndicator'))
  {
    refreshOpener(window);
  }    

  // Create a displayable variable list
  var vars = convertVariableIdsToDisplayable(document.getElementById("usedVariableIds").value, document.getElementById("changedVariableIds").value);
  document.getElementById("usedVariablesForDisplay").innerHTML = (vars=="") ? "<span class='softNone'>None</span>" : vars;

  // Create displayable prerequisites 
  var preReqs  = convertPreRequisiteIdsToDisplayable(document.getElementById("preRequisiteIds").value);
  document.getElementById("preRequisitesForDisplay").innerHTML = (preReqs=="")  ? "" : preReqs;

  if (rapTest) alert("exitbuildSelectors");
}


// Sets titles for components
function setComponentTitles() {
  var tdTags = document.getElementsByTagName("TD");
  for (var i=0; i<tdTags.length; i++) {
    var tagId = tdTags[i].id;
    if (tagId.substr(0,12)=="componentTD:") {
      var compTip = new Array();
      var segments = ComponentSegmentsMap[tagId.split(":")[1]];
      if (segments) {
        for (var s=0; s<segments.length; s++) {
          compTip[s] = SegmentLabelMap[segments[s]];
        }
      }
      else
        compTip[0] = "None Applicable";
      compTip.unshift("\nSegments:");
      var fldList = tagId.split(":")[2].split(" ");
      for (var f=0; f<fldList.length; f++) {
        compTip.unshift(fldList[f] + "=" + tagId.split(":")[3]);
      }
      compTip.unshift("Key Fields:");
      tdTags[i].title = compTip.join("\n");
    }
  }
}

// Sets the selected ruleTypeId in the form and re-populates selections
function changeRuleTypeSelector()
{
    document.ruleForm.ruleTypeId.value = document.ruleForm.ruleTypeIdSelector.value;
    if (document.ruleForm.queueId && document.ruleForm.queueId > 0) // S0864769
        document.ruleForm.queueId.selectedIndex = 0;                // S0864769
    if (document.ruleForm.alertType)                                // S0864769
        document.ruleForm.alertType.selectedIndex = 0;              // S0864769
    clearExpansionCache();
    loadFieldExpansions();
    loadMacroExpansions();
    loadOverviewExpansions();
    loadPrereqExpansions();
}



// Return comparison of last word in array element (for sort by value in function generateSortedOptions)
function valueComparator( inVal1, inVal2 )
{
    var parts = inVal1.split(" ");
    var outVal1 = parts[parts.length-1];
    parts = inVal2.split(" ");
    var outVal2 = parts[parts.length-1];
    return (outVal1 < outVal2) ? -1 : ((outVal1 == outVal2) ? 0 : 1);
}

// Sort a list of choices and create options for a selector from it
function generateSortedOptions(optx, selector, options, firstText, selection, sortByValue)
{
    if (rapTest)
        alert('entergenerateSortedOptions' + optx+","+selector.id+","+options.length+","+
               firstText+","+(selection?selection.value:null));

    // Sort the choices
    if (sortByValue)
        options.sort(valueComparator);
    else
        options.sort();

    // Create the instuctional first choice
    if (firstText)
        selector.options[optx++] = new Option(firstText,"");

    // Generate an Option for each choice (key value is last word in choice text)
    for (var i = 0; i < options.length; i++) 
    {
        // Peel key value off of choice text (far-right value)
        var parts = options[i].split(" ");
        var value = parts[parts.length-1];
        var label = options[i].substring(0, options[i].length-value.length);
        selector.options[optx++] = new Option(label,value);

        // Set the selection index if it matches the selection
        if (selection && selection.value == value)
            selector.selectedIndex = optx-1;
    }
    if (selection && selector.selectedIndex==0)
        selection.value = "";

    // Return current index in case there are more options coming
    if (rapTest) alert('exitgenerateSortedOptions' + optx);
    return optx;
}



// Display one of two displays per setting of toggle button
// When the button is 'selected', the gif for it contains 'selected'
function toggleButton(toggle, title1, title2, content1, content2 ) {
    // Turn everything off at first
    if ( content1 )
        document.getElementById( content1 ).style.display = 'none';
    if ( content2 )
        document.getElementById( content2 ).style.display = 'none';

    // Turn on only the selected area
    var srcParts = toggle.src.split("/");
    if (srcParts[ srcParts.length-1 ].indexOf( "selected" ) < 0)
    {
        if ( content1 )
            document.getElementById( content1 ).style.display = '';
        toggle.title = title1;
        srcParts[ srcParts.length-1 ] = "selected_" + srcParts[ srcParts.length-1 ];
    }
    else
    {
        if ( content2 )
            document.getElementById( content2 ).style.display = '';
        toggle.title = title2;
        srcParts[ srcParts.length-1 ] = srcParts[ srcParts.length-1 ].substring(9);
    }
    toggle.src = srcParts.join("/");
}



// Toggles the display of test results based on a radio button setting
function toggleRadio( tag, area1, area2 )
{
    document.getElementById(area1).style.display='';
    document.getElementById(area2).style.display='none';
    return true;
}


// Builds new Options for the field selector
function generateFieldOptions(optx, segments, selector, msg)
{
    if (rapTest) alert("generateFieldOptions");

    // Get lists of blank-separated field IDs
    var fields = new Array();
    for (var i=0; i<segments.length; i++) {
        fields[i] = SegmentFieldsMap[segments[i]].join(" ");
    }

    // Concatenate lists of field IDs, then split into individual IDs
    fields = fields.join(" ").split(" ");

    // Update the list of all fields with their selection text plus actual name as the value
    for (var i=0; i<fields.length; i++) {
        fields[i] = FieldLabelMap[fields[i]] + " " + FieldLabelMap[fields[i]].split(":")[0].replace(/^\s*|\s*$/g,"");
    }

    // Sort and create the options (key value is tacked on the end of each text string)
    optx = generateSortedOptions(optx, selector, fields, msg);

    if (rapTest) alert("exitgenerateFieldOptions");
    return optx;
}



// Returns an array of segment IDs present in both auth and query rule types for a transaction (optional)
// This code depends on the transaction acronym being the first 4 chars of the transaction label
// and the first char being "R" or "C" for the transaction type
function getAuthQuerySegments(optTranId) {
    var acronymSuffix = (optTranId==null) ? "" : TransactionLabelMap[optTranId].substring(1,4);
    var commonSegments = new Array();
    var comx = 0;
    for (var segId in SegmentLabelMap) {
        if (SegmentLabelMap[segId] instanceof Function) continue;
        var inA = false;
        var inQ = false;
        for (var tranId in TransactionSegmentsMap) {
            if (TransactionSegmentsMap[tranId] instanceof Function) continue;
            if (acronymSuffix != TransactionLabelMap[tranId].substring(1,1+acronymSuffix.length))
                continue;
            var segsList = " " + TransactionSegmentsMap[tranId].join(" ") + " ";
            if (segsList.indexOf(" "+segId+" ") >= 0) {
                var type = TransactionLabelMap[tranId].substring(0,1);
                if (type=="R")
                  inA = true;
                else if (type=="C")
                  inQ = true;
            }
            if (inA && inQ) {
                commonSegments[comx++] = segId;
                break;
            }
        }
    }
    return commonSegments;
}




// Returns an array of all segments in transactions starting with a specified prefix
// This code depends on "R" or "C" being the first char of the transaction label for the transaction type
function getSegmentsForPrefix(prefix) {
    if (rapTest) alert("getSegmentsForPrefix " + prefix);
    var segmentsArray;
    var segmentsMap = new Array();
    for (var tranId in TransactionLabelMap) {
        var tranLabel = TransactionLabelMap[tranId];
        if (tranLabel instanceof Function) continue;
        // For each transaction starting with the prefix ("R" or "C"), add its segment IDs to the segmentsMap
        if (tranLabel.substring(0,1)==prefix) {
            segmentsArray = TransactionSegmentsMap[tranId]; // .split(" ");
            for (var i=0; i<segmentsArray.length; i++) {
                segmentsMap[segmentsArray[i]] = segmentsArray[i];
            }
        }
    }
    segmentsArray = new Array();
    var i = 0;
    for(var sm in segmentsMap) {
        if (segmentsMap[sm] instanceof Function) continue;
        segmentsArray[i++] = sm;
    }
    if (rapTest) alert("exitgetSegmentsForPrefix " + segmentsArray);
    return segmentsArray;
}


// Gets a ruleType code for the selected ruleType ID
function getRuleType()
{
    if (rapTest) alert("getRuleType");
    var ruleTypeId = document.ruleForm.ruleTypeId.value;
    var ruleType = ruleTypeCodeMap[ruleTypeId];    
    var rval = (!ruleType || (ruleType == '') ) ? "Q" : ruleType;
    if (rapTest) alert("exitgetRuleType " + ruleType + "  RuleTypeId" + ruleTypeId + " RuleTypeEval " + (!ruleType) + " Rval " + rval);
    return rval;
}


// Adds the text to the SAS source code area at the cursor
function addText(textToAdd)
{
    textToAdd = " " + textToAdd + " ";
    var textArea = document.ruleForm.code;
    if (document.selection)   // IE
    { 
        textArea.focus();
        var range = document.selection.createRange();
        range.text = textToAdd;
        // Highlight inserted text 
        range.moveStart('character', -textToAdd.length);
        range.select();
    }

    else if (textArea.selectionStart || textArea.selectionStart == '0')   // FF
    {
        var startPos = textArea.selectionStart;
        var endPos = textArea.selectionEnd;
        textArea.value = textArea.value.substring(0, startPos) + textToAdd + textArea.value.substring(endPos);
        // Highlight inserted text
        textArea.setSelectionRange(startPos,startPos+textToAdd.length);
    }

    else     // Neither IE nor Firefox
    {
        textArea.value += textToAdd;
    }
    textArea.focus();
}



// Sets values when resultsTestContent.jsp finishes loading
//r HIGHTABS = "back highlinkview notetranview linktranview ruleview notetextview notebridview rulelinkview highruleview marks notes directs values expressions templates resolveds".split(" ");
var HIGHTABS = "back notetranview highruleview values templates directs resolveds".split(" ");
function showTestResultsLoaded( resultsCount )
{
    if (rapTest) alert('showTestResultsLoaded')

    // Update status field at top of containing tab
    if (resultsCount || resultsCount==0) 
        setErrorWarningDisplay(parent.document, 'testResults', 'Test Results', '<font color=blue><b>'+resultsCount+' Results&nbsp;&nbsp;&nbsp;');

    // Display extra buttons if queuing rule
    var ruleTypeInfo = parent.document.getElementById("ruleTypeInfo");

    if (ruleTypeInfo && (ruleTypeInfo.innerHTML.replace(/^\s*|\s*$/g,"")=="Queue" || ruleTypeInfo.innerHTML.replace(/^\s*|\s*$/g,"")=="Authorization"))
    {
        var buttons;
        var alertType = parent.document.getElementById('alertType');
        var highlightAlertType = parent.document.getElementById('highlightAlertType');
        if (highlightAlertType && alertType && alertType.options[alertType.selectedIndex].value==highlightAlertType.value)
            buttons = HIGHTABS;
        else
            buttons = [ "alerts","back" ];
        for (var b=0; b < buttons.length; b++) {
           var button = parent.resultsFrame.document.getElementById(buttons[b] +"Button");
           if (button) button.style.display = "";
           button = parent.resultsFrame.document.getElementById(buttons[b]+"ButtonRefresher");
           if (button) button.style.display = "";
        }
    }

//
// TODO: Perhaps reinstate this button if folks insist they want better summarization from rule testing
//    // Display button for summary table if not variable rule
//    if (ruleTypeInfo && ruleTypeInfo.innerHTML.replace(/^\s*|\s*$/g,"")!="Variable")
//    {
//        var button = document.getElementById("summaryButton");
//        if (button) button.style.display = "";
//        button = document.getElementById("summaryButtonRefresher");
//        if (button) button.style.display = "";
//    }

    if (rapTest) alert('exitshowTestResultsLoaded')
}



// Returns the list of all fields
var allFieldNames;
function getAllFieldNames() {
    if (!allFieldNames || allFieldNames.length<=1)
    {
        var resultsFrame = (resultsFrame) ? resultsFrame : self;
        var list = resultsFrame.document.getElementById("allFieldNames").value;
        allFieldNames = ( list && list!=""
                        ? list.split(" ")
                        : new Array() );
    }
    return allFieldNames;
}


// Shows or hides table cells per 'shownVarsMap' contents
function showHideCells(shownVarsMap, row, tagName)
{
    var allNames = getAllFieldNames();
    var cells = row.getElementsByTagName(tagName);
    for (var f=0; f<allNames.length; f++)
    {
        if (shownVarsMap[allNames[f]])
            cells[f].style.display = "";
        else
            cells[f].style.display = "none";
    }
}



// Causes a column in the results table to appear
function showField(fieldName, fieldNumber)
{
    showHideField('Show', fieldNumber);
}



// Hides a field or segment based on whether ctrl or shift was held down for click
function hideSegmentOrField(fieldName, fieldNumber)
{
    if (ctrlPressed || shiftPressed)
        hideSegment(fieldName);
    else
        hideField(fieldName, fieldNumber);
}





// Causes a column in the results table to disappear
function hideField(fieldName, fieldNumber)
{
    showHideField('Hide', fieldNumber);
}




// Causes a column in the results table to appear or disappear
function showHideField(showHide, fieldNumber)
{
    var displaySetting = (showHide.indexOf("Show")==0 ? "" : "none");

    // Access the results rows for the table
    var resultsFrame = (resultsFrame) ? resultsFrame : self;
    var rows = resultsFrame.document.getElementsByTagName("TR");

    // Loop through the header and all data columns to show the field
    for (var r=0; r<rows.length; r++)
    {
        var row = rows[r];
        if (row.id == "tableTitle")
            ;
        else if (row.id == "columnHeaders")
            row.getElementsByTagName("TH")[fieldNumber].style.display = displaySetting;
        else
            row.getElementsByTagName("TD")[fieldNumber].style.display = displaySetting;
    }
}


// Causes all columns in the field's segment in the test results table to appear
function showSegment(fieldName)
{
    if (!fieldName || fieldName=="")
        return;
    showHideSegment('Show', fieldName);
}



// Causes all columns in the field's segment in the test results table to disappear
function hideSegment(fieldName)
{
    showHideSegment('Hide', fieldName);
}



// Causes all columns in the field's segment in the test results table to appear or disappear
function showHideSegment(showHide, fieldName)
{
    var segmentName = (fieldName+"_").split("_")[0];
    var allNames = getAllFieldNames();
    for (var f=0; f<allNames.length; f++)
    {
        if (allNames[f].substring(0,segmentName.length) == segmentName) {
            showHideField(showHide, f);
        }
    }
}




// Sets values when saslogContent.jsp finishes loading
function showSasLogLoaded() {

    var parentForm = parent.document.forms[0];
    if (parentForm && parentForm.usedVariables && parentForm.changedVariables && parentForm.preRequisites && parentForm.errorCount && parentForm.warningCount && parent.resultsFrame)
    {
        var obsCount = document.logForm.obsCount.value;
        var resultsCount = document.logForm.resultsCount.value;
        var errorCount = document.logForm.errorCount.value;
        var warningCount = document.logForm.warningCount.value;

        if (errorCount == "0") {
            parentForm.generatedCode.value = document.logForm.generatedCode.value;

            var usedVars = document.logForm.usedVariables.value;
            parentForm.usedVariables.value = usedVars;
            var changedVars = document.logForm.changedVariables.value;
            parentForm.changedVariables.value = changedVars;
            var vars = parent.convertVariableIdsToDisplayable(parent._convertVariableNamesToIds(usedVars), parent._convertVariableNamesToIds(changedVars));
            vars = (vars.replace(/^\s*|\s*$/g,"")=="") ? "<span class='softNone'>None</span>" : vars;
            if (parent.document.getElementById("usedVariablesForDisplay").innerHTML != vars)
                parent.document.getElementById("usedVariablesForDisplay").innerHTML = vars;
            parent.document.getElementById("weakRequisitesForDisplay").innerHTML = document.getElementById("weakRequisitesForDisplay").innerHTML;
            parent.document.getElementById("dependencySeparator").innerHTML = document.getElementById("dependencySeparator").innerHTML;

            var preReqs = document.logForm.preRequisites.value;
            parentForm.preRequisites.value = preReqs;
            preReqs = (preReqs.replace(/^\s*|\s*$/g,"")=="") ? "" : parent.convertPreRequisiteIdsToDisplayable(preReqs);
            if (parent.document.getElementById("preRequisitesForDisplay").innerHTML != preReqs)
                parent.document.getElementById("preRequisitesForDisplay").innerHTML = preReqs;
        } else if (errorCount>0)
        { 
            parent.document.getElementById("usedVariablesForDisplay").innerHTML = "<i>? (See errors in rule code)</i>";
            parent.document.getElementById("preRequisitesForDisplay").innerHTML = "<i>? (See errors in rule code)</i>";
            parent.document.getElementById("dependencySeparator").innerHTML = "";
            parent.document.getElementById("weakRequisitesForDisplay").innerHTML = "";
        }

        if (errorCount != "")
        {
            // Pass error, warning, and result counts to parent for easy reference
            parentForm.errorCount.value = errorCount;
            parentForm.warningCount.value = warningCount;
    
            // Replace spinner with appropriate message for SAS run
            var title = " title='SAS log contains " + errorCount + " errors and " + warningCount + " warnings'";
            var message;
            var targetTab = 'sasLog';
            var targetLab = 'SAS Log';
            if (errorCount != 0)
                message = "<font color='red'" + title + "><b>" + errorCount + " Errors!&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b></font>";
            else if (warningCount != 0)
                message = "<font color='green'" + title + "><b>" + warningCount + " Warnings!</b></font>";
            else if (!resultsCount || resultsCount == 0)
                message = "<font color='blue'><b>Success!&nbsp;&nbsp;&nbsp;</b></font>";
            else
            {
                message = "<font color='blue'><b>" + resultsCount + " Results&nbsp;&nbsp;&nbsp;</b></font>";
                targetTab = "testResults";
                targetLab = "Test Results";
            }

            // Copy error indicators into the gutter next to the code textarea 
            parentGutter = parent.document.getElementById("gutter");
            if (parentGutter)
            {
               parentGutter.innerHTML = document.getElementById("gutter").innerHTML;
               parentCode = parent.document.getElementById("code");
               if (parentCode)
                  parentGutter.scrollTop = parentCode.scrollTop;
            }

            setErrorWarningDisplay( parent.document, targetTab, targetLab, message );
        }

        // Replace any previously-obtained results with a "Refresh" button
        setProperty( parent.resultsFrame.document.getElementById("actualResults"), 'style.display', 'none' );
        setProperty( parent.resultsFrame.document.getElementById("resultsRefresher"), 'style.display', '' );
        showTestResultsLoaded();

        // If this was just a codegen request, display it now
        if (document.logForm.codeGenOnly.value == 'true')
           parent.window.showGeneratedCode();
    }
}



function setProperty( tag, property, value)
{
    if (tag)
        eval("tag." + property + "=value");
}



// Forces reload of summary data and submits request
function submitResultsRequestAndReload( tableType, rowCount, whereClause, target )
{
    var resForm = target ? document.popupResultsForm : document.resultsForm;        
    resForm.queFiredTransactionCount.value = '';
    resForm.ardFiredTransactionCount.value = '';
    submitResultsRequest( tableType, rowCount, whereClause, target );
}

// Copies parms to the results jsp and submits its form
function submitResultsRequest( tableType, rowCount, whereClause, target )
{
    if (rapTest) alert('submitResultsRequest ' + tableType);
    //alert(tableType+" rowCount="+rowCount+" whereClause="+whereClause+" target="+target);

    // If no SAS operation in progress, submit the request
    if ( !isSasActive('complain') )
    {
        // Copy info from the parent form into the resultsForm fields
        var resForm = target ? document.popupResultsForm : document.resultsForm;        
        resForm.baseRuleId.value = getTagValue( parent.document.ruleForm.baseRuleId );
        resForm.version.value = getTagValue( parent.document.ruleForm.version );
        resForm.obsCount.value = getTagValue( parent.document.ruleForm.obsCount );
        resForm.alertType.value = getTagValue( parent.document.ruleForm.alertType );
        resForm.resultsRuleType.value = getTagHTML( parent.document.getElementById("ruleTypeInfo") );
        resForm.usedVariablesForDisplay.value = getTagHTML( parent.document.getElementById("usedVariablesForDisplay") );
        resForm.firstRow.value = 1;
        
        if (resForm.rowCount)
        {
          if (rowCount)
             resForm.rowCount.value = rowCount;
          else 
             resForm.rowCount.value = -1; // negative number forces default value
        }
        resForm.byVars.value = "";
        
        // Display the spinner while we wait if targetting this window        
        if (!target) {
            setErrorWarningDisplay( parent.document, 'testResults', 'Test Results', '<image title="Loading test results..." src="' + sasSpinner + '" border="0" height="15px" width="15px"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' );
            resForm.target = '_self';
        }
        else
            resForm.target = target;

        if (whereClause) 
          resForm.whereClause.value = whereClause;
        else
          resForm.whereClause.value = "";
  
        // Tell the action class which table to read
        resForm.tableType.value = tableType;

        // Submit the request for the table
        resForm.submit();
    }

    if (rapTest) alert('exitsubmitResultsRequest');
}

// Update the display of certain General Properties upon changes
function resizeProperties()
{
   var queueRow = document.getElementById("queueRow");
   var alertTypeRow = document.getElementById("alertTypeRow");
   var alertReasonRow = document.getElementById("alertReasonRow");

   // For variable rules and possibly ARD rules, hide queue selector, level selector, and reason textarea
   var rt = getRuleType().substr(0,1);
   if (rt=='Q' || (rt=='A' && document.getElementById("oseAlertInd") && document.getElementById("oseAlertInd").value=='true'))
   {
      queueRow.style.display = "";
      alertTypeRow.style.display = "";
      alertReasonRow.style.display = "";
   }
   else
   {
      queueRow.style.display = "none";
      alertTypeRow.style.display = "none";
      alertReasonRow.style.display = "none";
   }
}


// Clears out instructional comment from priority box upon focus
function focusExecutionPrioritySelector()
{
  var orderPriority = document.getElementById("executionOrderPriority");
  if (orderPriority.value==document.getElementById("prty").value)
     orderPriority.value = '';
}


// Controls display of priority value box based on priority type selection
function changeExecutionPrioritySelector()
{
   var xeqTypeTD = document.getElementById("xeqTypeTD");
   var xeqPrtyTD = document.getElementById("xeqPrtyTD");
   var orderPrty = document.getElementById("executionOrderPriority");
   var orderType = document.getElementById("executionOrderType");
   if (orderType.value==1 || orderType.value==-1)
   {
      if (xeqPrtyTD.style.display=='none')
      {
         xeqPrtyTD.style.display='';
         orderType.style.width = orderType.offsetWidth - orderPrty.offsetWidth;
         xeqTypeTD.colSpan='1';
      }
   }
   else if (xeqPrtyTD.style.display!='none')
   {
      var prtyLeft = getLeftTop(orderPrty)[0];
      var typeLeft = getLeftTop(orderType)[0];
      var prtyWidth = orderPrty.offsetWidth;
      xeqPrtyTD.style.display='none';
      xeqTypeTD.colSpan='2';
      orderType.style.width = (prtyLeft - typeLeft + prtyWidth) + 'px';
   }   
}


// Finds a string in the SAS code (generated or entered version)
function findCode(prefix, previous)
{
    var area = document.getElementById('code');
    findString( eval('document.ruleForm.'+prefix+'_stringToFind'),
                area,
                'match_case_button',
                eval('document.ruleForm.'+prefix+'_stringToFind.value'),
                previous );
}

// Replaces a string in the SAS code (entered version only)
function replaceCode(prefix, findString, previous)
{
    replaceString( eval('document.ruleForm.'+prefix+'_stringToFind'),
                   document.getElementById('code'),
                   'match_case_button',
                   eval('document.ruleForm.'+prefix+'_stringToSubstitute.value'),
                   (findString) ? eval('document.ruleForm.'+prefix+'_stringToFind.value') : null,
                   previous );
}



// Send a URL to the server in the background and do the callback when it is done
var IFrameObj; // our IFrame object
function runInBackground(url, callback)
{
  if (!document.createElement) {return true};
  var IFrameDoc;
  var URL = 'server.html';
  if (!IFrameObj && document.createElement) {
    // create the IFrame and assign a reference to the
    // object to our global variable IFrameObj.
    // this will only happen the first time 
    // callToServer() is called
   try {
      var tempIFrame=document.createElement('iframe');
      tempIFrame.setAttribute('id','RSIFrame');
      tempIFrame.style.border='0px';
      tempIFrame.style.width='0px';
      tempIFrame.style.height='0px';
      IFrameObj = document.body.appendChild(tempIFrame);
      
      if (document.frames) {
        // this is for IE5 Mac, because it will only
        // allow access to the document object
        // of the IFrame if we access it through
        // the document.frames array
        IFrameObj = document.frames['RSIFrame'];
      }
    } catch(exception) {
      // This is for IE5 PC, which does not allow dynamic creation
      // and manipulation of an iframe object. Instead, we'll fake
      // it up by creating our own objects.
      iframeHTML='\<iframe id="RSIFrame" style="';
      iframeHTML+='border:0px;';
      iframeHTML+='width:0px;';
      iframeHTML+='height:0px;';
      iframeHTML+='"><\/iframe>';
      document.body.innerHTML+=iframeHTML;
      IFrameObj = new Object();
      IFrameObj.document = new Object();
      IFrameObj.document.location = new Object();
      IFrameObj.document.location.iframe = document.getElementById('RSIFrame');
      IFrameObj.document.location.replace = function(location) {
        this.iframe.src = location;
      }
    }
  }
  
  if (navigator.userAgent.indexOf('Gecko') !=-1 && !IFrameObj.contentDocument) {
    // we have to give NS6 a fraction of a second
    // to recognize the new IFrame
    setTimeout('callToServer()',10);
    return false;
  }
  
  if (IFrameObj.contentDocument) {
    // For NS6
    IFrameDoc = IFrameObj.contentDocument; 
  } else if (IFrameObj.contentWindow) {
    // For IE5.5 and IE6
    IFrameDoc = IFrameObj.contentWindow.document;
  } else if (IFrameObj.document) {
    // For IE5
    IFrameDoc = IFrameObj.document;
  } else {
    return true;
  }
  
  IFrameDoc.location.replace(URL);
  return false;
}


// Appends a message to the debug log for the page (if visible)
function log(line)
{
  var log = document.getElementById("debugLog");
  if (!log)
      return;
  if (log.style.height=='0px') {
      log.style.display = '';
      log.style.height = '100px';
      log.style.width = '500px';
  }
  log.innerHTML += line + "<br>";
}

// Clears the debug log for the page 
function clearLog(line)
{
  var log = document.getElementById("debugLog");
  if (log)
    log.innerHTML = "";
}

// Refreshes the page that opened this page if it is showing Coding rules
function refreshOpener(theWindow)
{
  try {
    if (theWindow.opener.parent &&
        theWindow.opener.parent.frames['overview'] &&
        theWindow.opener.parent.frames['overview']._rulesFolderTree &&
        theWindow.opener.parent.frames['overview']._rulesFolderTree.getSelectedNodes() &&
        theWindow.opener.parent.frames['overview']._rulesFolderTree.getSelectedNodes()[0].parentId == 'Coding' 
        )      
    {      
      theWindow.opener.window.location.reload();
    }
  } catch (error) {
     // If the opener moves away from the original rule list, we get an exception (ignore it)
  }


  
}



// Toggle all action selections
function actionToggleAll(box,disable)
{
    var allChecked = box.checked;
    var tags = document.getElementsByName('action_checkbox');
    for (var i=0; i<tags.length; i++) {
        tags[i].checked = allChecked;
        if (disable)
            tags[i].disabled = true;
    }
    if (allChecked)
        document.getElementById('actionCommands').value = "ALL";
    else
        document.getElementById('actionCommands').value = "";
}

// Record action checkbox selections
function selectActionRow(box,rowId) {
    var actions = document.getElementById('actionCommands');
    if (box.checked)
        actions.value += ',' + rowId;
    else {
        actions.value += ',(' + rowId + ')';
        var t = document.getElementById('action_all_checkbox');
        if (t) t.checked = false;
    }
}

// Do the selected actions for the selected rows
function applyActions() {
    if (document.getElementById("action_all_checkbox").disabled) {
        alert("Apply filter changes before requesting alert updates.");
        return;
    }
    document.resultsForm.firstRow.value = '1';
    document.resultsForm.executeActionCommands.value = 'true';
    setBusyDetail();
    document.resultsForm.submit();
}

// Run the commands that set the checkboxes
function playbackActionCommands() {
    var cmdTag = document.getElementById('actionCommands');
    if (!cmdTag)
        return;

    var actions = cmdTag.value.split(',');        
    for (var a=0; a<actions.length; a++) {
        var action = actions[a];
        if (action=='')
            continue;
        else if (action=='ALL') {
            var box = document.getElementById('action_all_checkbox');
            if (box) box.checked = true;
            var tags = document.getElementsByName('action_checkbox');
            if (tags) {
                for (var t=0; t<tags.length; t++) {
                    tags[t].checked = true;
                }
            }
        }
        else {
            var box = document.getElementById('action_checkbox_' + eval(action));
            if (box)
                box.checked = !(action.substring(0,1)=='(');
        }
    }
}



// Hides all filterers
function hideAllFilterers() {
  var filterers = document.getElementsByTagName("DIV");
  for (var i=0; i<filterers.length; i++) {
    var id = filterers[i].id;
    if (id && id.indexOf("Filterer")==id.length-8 && filterers[i].style.left!='-10000px')
      showHideFilterer(id.substr(0,id.length-8));
  }
}

// Toggles display of the widjet we use to filter data
function showHideFilterer(column) {
  var filterer = document.getElementById(column+"Filterer");
  var filtererStyle = filterer.style;
  if (filtererStyle.left=='-10000px') {
    hideAllFilterers();
    var head = document.getElementById('head_'+column);
    var headWidth = head.offsetWidth + 32;
    var headLeft = head.offsetLeft;    
    var filtererWidth = filterer.offsetWidth + 24;    
    var headersWidth = document.getElementById('columnHeaders').offsetWidth;
    var xPos = headLeft;
    if (xPos + filtererWidth > headersWidth)
        xPos = headersWidth - filtererWidth;
    if (xPos < 10)
        xPos = 10;
    //alert('headWidth='+headWidth +' filtererWidth='+filtererWidth+' headersWidth='+headersWidth+' clientX='+event.clientX + " xPos=" + xPos + ' headersWidth=' + headersWidth);        
    filtererStyle.left = xPos + 'px';
    filtererStyle.top = (event.clientY) + 'px';    
  } else {
    filtererStyle.top='100px';
    filtererStyle.left='-10000px';
  }
}

// Toggle all selections in a filter
function filterToggleAll(box,tagName)
{
    var allChecked = box.checked;
    var tags = document.getElementsByName(tagName);
    for (var i=0; i<tags.length; i++)
        tags[i].checked = allChecked;
    updateFilterCorral(tagName);
}

// Reveal button that will apply filters
function showFilterChanged() {
    var actionCommands = document.getElementById('actionCommands');
    if (actionCommands) {
        actionCommands.value = '';
        var box = document.getElementById("action_all_checkbox");
        box.checked = false;
        actionToggleAll(box,true);
        box.disabled = true;
    }
    document.getElementById('filterLabel').style.display = "none";
    document.getElementById('filterChangedLabel').style.display = "";
}

// Update display of active filters
var FILTERS_NONE = "None";
var FILTERS_LONGEST = 256;
function updateFilterCorral(tagName)
{
        
    // If non-specific request, create corral from scratch
    if (!tagName) {
        var prevCorral = document.getElementById("previousFilterCorral");
        if (prevCorral)
            prevCorral.value = '';
        var prevName = "";
        var inputTags = document.getElementsByTagName("INPUT");
        for (var i=0; i<inputTags.length; i++) {
            if (!inputTags[i] || !inputTags[i].name)
                continue;
            var name = inputTags[i].name;
            if (name!=prevName && name.indexOf('filterCheck_')==0 || name.indexOf('filterText_')==0)
                updateFilterCorral(name);
            prevName = name;
        }
        return;
    }
    
    // Get values for this tag in an array
    var tagValues = new Array();
    var tags = document.getElementsByName(tagName);
    var tagLabel;
    if (tagName.indexOf('filterText_')==0) {
        tagLabel = getLabelFromColumnName(tagName.substring(11));
        if (tags[0].value && tags[0].value.replace(/^\s*|\s*$/g,"")!='') {
            tagValues.push(escapeXml(tags[0].value));
        }
    }
    else if (tagName.indexOf('filterCheck_')==0) {
        tagLabel = getLabelFromColumnName(tagName.substring(12));
        for (var i=0; i<tags.length; i++) {
            if (tags[i].checked) {
                tagValues.push(escapeXml(tags[i].value));
            }
        }
    }
    else
        return;

    // Update the values for this tag in the corral
    tagLabel = '<B>' + tagLabel + '</B>'; 
    var corral = document.getElementById("filterCorral");
    var filters = (corral.innerHTML==FILTERS_NONE || corral.innerHTML=='') ? new Array() : corral.innerHTML.split("<BR>");
    var addFilter = true;
    for (var i=0; i<filters.length; i++) {
        var filterLabel = filters[i].split(' = ')[0];
        if (filterLabel == tagLabel) {
            if (tagValues.length>0) {
                var vals = tagValues.join(", ");
                filters[i] = tagLabel + ' = ' + (vals.length>FILTERS_LONGEST ? vals.substring(0,FILTERS_LONGEST-1)+'...' : vals);
                addFilter = false;
                break;
            }
            else {
                filters.splice(i,1);
                addFilter = false;
                break;
            }
        }
    }
    if (addFilter && tagValues.length>0) {
        var vals = tagValues.join(", ");
        filters.push(tagLabel + ' = ' + (vals.length>FILTERS_LONGEST ? vals.substring(0,FILTERS_LONGEST-1)+'...' : vals));
    }
    if (filters.length==0) {
        corral.innerHTML = FILTERS_NONE;
        document.getElementById("previousFilterCorral").value = FILTERS_NONE;
    }
    else {
        var joined = filters.join("<BR>");
        corral.innerHTML = joined;
        document.getElementById("previousFilterCorral").value = joined;
    }
}

// Clears all filters and redisplays data
function resetFilter() {
    document.getElementById("previousFilterCorral").value = "";
    filterResults();
}

// Submits a request to filter results
function filterResults()
{
    document.resultsForm.firstRow.value = '1';
    setBusyDetail();
    document.resultsForm.submit();
}

// Submits a request to create Excel spreadsheet
function createSpreadsheet()
{
    document.resultsForm.firstRow.value = '1';
    document.resultsForm.xls.value = 'true';
    document.resultsForm.submit();
    document.resultsForm.xls.value = '';        
}

// Toggles display of background color of column selections as mouse hovers
function filterUnhover(tag) {
  tag.style.backgroundColor='white';
}
function filterHover(tag) {
  tag.style.backgroundColor='lavender';
}

// Repositions selector per dragging mouse
filtererX=-1000;
filtererY=-1000;
filtererType='init';
function filterDragMouseDown(nameType) {
  if (filtererType='init') {
    document.body.onmousemove=filterDragMouseMove;
    filtererType='';
  }
  var selStyle = document.getElementById(nameType+'Filterer').style;
  filtererX=parseInt(selStyle.left)-event.clientX;
  filtererY=parseInt(selStyle.top)-event.clientY;
  filtererType=nameType;
}
function filterDragMouseUp() {
  filtererX=-1000;
  filtererY=-1000;
  filtererType='';
}
function filterDragMouseMove() {
  if (filtererX!=-1000 && filtererY!=-1000 && filtererType!='') {
    var selStyle=document.getElementById(filtererType+'Filterer').style;
    selStyle.top=(event.clientY+filtererY) + 'px';
    selStyle.left=(event.clientX+filtererX) + 'px';
  }
}





// Add indicated column to the list of byVars and refresh the list from row 1
function sortTestResults(column, resultsCount)
{
  if (resultsCount <= 1)
     return;
  var byVars = document.resultsForm.byVars.value.replace(/^\s*|\s*$/g,"").split(" ");
  if (byVars.length==0 || byVars[0]!=column)
  {
     var newByVar = new Array();
     newByVar[0] = column;
     newByVar[1] = "ASC";
     if (byVars.length > 1)
        newByVar[2] = ",";
     byVars = newByVar.concat(byVars);
     if (byVars.length > 14)
         byVars.length = 14; // retain no more than 5 sort terms
  }
  // if same column as before, just change direction of sort
  else
     byVars[1] = (byVars[1]=="DESC") ? "ASC" : "DESC";
  document.resultsForm.byVars.value = byVars.join(" ").replace(/^\s*|\s*$/g,"");
  document.resultsForm.firstRow.value = '1';
  document.resultsForm.submit();
}




// Show a named element safely, return element
function safeShow(id) {
  var tag = document.getElementById(id);
  if (tag) 
    tag.style.display = '';
  return tag;
}

// Hide a named element safely, return element
function safeHide(id) {
  var tag = document.getElementById(id);
  if (tag) 
    tag.style.display = 'none';
  return tag;
}


// Selects/unselects displayed components based on the set of componentIds passed in
function setComponentSelections(selectedCompIds, saveNotNecessary)
{
  if (!selectedCompIds)
    selectedCompIds = TransactionComponentsMap[document.getElementById("componentPreSetSelector").value];
  if (selectedCompIds==undefined)
    selectedCompIds = new Array();
  for (var compTypeCode in compTypeCodes) {
    for (var c=0; c<compTypeCodes[compTypeCode].length; c++) {
      var compSubTypeCode = compTypeCodes[compTypeCode][c];
      var allTag = document.getElementById("Checkbox"+compTypeCode+compSubTypeCode+"All");
      if (allTag)
        allTag.checked = false;
      for (var i=1; ;i++) {
        var trId = compTypeCode+compSubTypeCode+(i++);
        var trTag = document.getElementById(trId);
        if (!trTag) break;
        var inputTags = trTag.getElementsByTagName("INPUT");
        var compId = inputTags[0].value.split(':')[0];
        inputTags[0].checked = selectedCompIds[compId];
        if (inputTags[1]) {
           compId = inputTags[1].value.split(':')[0];
           inputTags[1].checked = selectedCompIds[compId];
        }
      }
    }
    setCompList(compTypeCode);
  }
  reorderPreSetSelections();
  if (!saveNotNecessary) enableSaveButton(event);
}


// Re-create cntl-space expansions based on new component selections
function setComponentExpansions() {
  clearExpansionCache();
  loadFieldExpansions();
  loadOverviewExpansions();
}


// Checks or unchecks all values of a component type (or subType)
function toggleAll(tag,code) {
  var inputTags = document.getElementsByTagName("INPUT");
  for (var inputTag in inputTags) {
    if (inputTags[inputTag].id && inputTags[inputTag].id.substr(0,10+code.length)=='component_'+code)
        inputTags[inputTag].checked = tag.checked;
  }
  changeCompList(code);
}

// Highlight onhover
function componentHover(tag) {
  if (tag.className.indexOf('Closed')>0)
    tag.className = 'componentClosedHeadHover';
  else
    tag.className = 'componentOpenHeadHover';
  //tag.style.textDecoration = 'underline';
}
// Remove highlight when not hovering
function componentUnhover(tag) {
  if (tag.className.indexOf('Closed')>0)
    tag.className = 'componentClosedHead';
  else
    tag.className = 'componentOpenHead';
  //tag.style.textDecoration = 'none';
}


// Changes display show/hide setting for component type details
function showHideComponent(compTypeCode,compSubTypeCode) {
  if (document.getElementById(compTypeCode + compSubTypeCode + "All").style.display=='none')
     setComponentDisplay(compTypeCode,"");
  else
     setComponentDisplay(compTypeCode,"none");
  componentUnhover(document.getElementById(compTypeCode + "ValueTD"));
}

// Sets display attibute for a component type
function setComponentDisplay(compTypeCode,disp) {
  // Show/hide component details
  var trs = document.getElementsByTagName("TR");
  for (var tr in trs) {
    if (trs[tr].id && trs[tr].id.substr(0,compTypeCode.length)==compTypeCode && trs[tr].id!=compTypeCode+"Head") 
        trs[tr].style.display=disp;
  }

  // Flip expand/collapse button and header colors
  var imUp = document.getElementById(compTypeCode+"ImgUp");
  var imDn = document.getElementById(compTypeCode+"ImgDn");
  var vals = document.getElementById(compTypeCode+"Values");
  var valTD = document.getElementById(compTypeCode+"ValueTD");
  if (disp=='none') {
    imUp.style.display = 'none';
    imDn.style.display = '';
    //valTD.title = "Click to show or hide component choices";
    //vals.className = "componentValuesClosed";
    valTD.className = 'componentClosedHead';
  }
  else {
    imUp.style.display = '';
    imDn.style.display = 'none';
    //vals.className = "componentValuesOpen";
    valTD.className = 'componentOpenHead';
  }
}
   
// Checks or unchecks a component selection
function toggleCheckbox(id) {
  var tag = document.getElementById(id);
  tag.checked = !tag.checked; 
}

// Changes components and sets the display list
function changeCompList(compTypeCode) {
  enableSaveButton(event);
  setCompList(compTypeCode);
  // Show pre-set no longer selected
  document.getElementById("componentPreSetSelector").selectedIndex=0;
}

// Sets the compact list of component values based on which ones are checked
function setCompList(compTypeCode) {
  compTypeCode = compTypeCode.split('_')[0]; // Strip any subType
  var compList = new Array();
  var inputTags = document.getElementsByTagName("INPUT");
  for (var i=0; i<inputTags.length; i++) {
    if (inputTags[i].id && inputTags[i].id.substr(0,10+compTypeCode.length)=="component_"+compTypeCode && inputTags[i].checked) 
      compList.push(inputTags[i].value.split(":")[2]);
  }
  document.getElementById(compTypeCode+"Values").innerHTML = compList.length==0 ? "<I>"+NOT_RELEVANT_TEXT+"</i>" : compList.join(", ");
  setComponentExpansions();
}


// Convert a column name to a label (see com.sas.finserv/creditfraud/ags/util/StringUtils.java)
function getLabelFromColumnName(columnName) {
    if (!columnName)
        return "";
    var tokens = columnName.split("_");
    var sb = new Array();
    for (var i=0; i<tokens.length; i++)
    {
        var tok = tokens[i].toLowerCase();
        var word = "";
        if (i==0 && tok.length==3 && tok!="raw" && tok!="lst")
            word = tok.toUpperCase();
        else if (tok=="id" || tok=="pb" || tok=="ob")
            word = tok.toUpperCase();
        else if (tok.length > 0)
            word = tok.substring(0,1).toUpperCase() + (tok.length>0 ? tok.substring(1) : "");
        sb.push(" " + word);
    }
    return sb.join("").replace(/^\s*|\s*$/g,"");
}


// Pagination controls
function pageToRow(row) {
    var lab = document.getElementById('filterChangedLabel');
    if (lab && lab.style.display == '')
        document.resultsForm.firstRow.value = 1;
    else
        document.resultsForm.firstRow.value = row;
    document.resultsForm.submit()    
}

// Drill to popup details on highlights/notes
function drillToRuleTestPopup(col, count, tableType, whereTemplate, whereArgument1, whereArgument2) {
    if (count==0) {
        alert("Nothing to display (" + col + "=0)");
    }
    else {
        //alert(col + '\n' + count + '\n' + tableType + '\n' + whereTemplate + '\n' + whereArgument1 + '\n' + whereArgument2);
        var whereClause = whereTemplate.replace('#',whereArgument1);
        whereClause = ' where ' + whereClause.replace('#',whereArgument2);
        var popName = "RulesStudioResultsPopup";
        var popWindow = open("",popName,'resizable=1,scrollbars=1,status=1,width=650,height=450,left=' + (parent.screenLeft+200) + ',top=' + (parent.screenTop+65));
        
        popWindow.focus();
        submitResultsRequest(tableType, count, whereClause, popName);
    }
}
