CREATE OR REPLACE PROCEDURE DELETE_XACTIONS (
   I_job_id         IN NUMBER,
   I_job_run_id     IN NUMBER,
   I_TABNAME        IN VARCHAR2,
   I_COMMIT_POINT   IN NUMBER,
   I_START_DATE     IN TIMESTAMP)
-----------------------------------------------------------------------------
-- File:  DELETE_XACTIONS.SQL
--
--
-- Description:
--     Delete transactions from the SOR database transaction tables.
--     Commit every i_commit_point rows to control the redo generation.
--
--
--
-- Called By:
--      SAS Batch Job
--
-- Calls:
--     N/A
--
--
-- Notes:
--
--
-- Usage:
--     db2 "call DELETE_XACTIONS(50100,50100'FCM_CCCA', 10000, '12-Jan-01')"
--
--
-- History:
--     Date          User             Notes
--     2012/07/23    leushe              Ported from DB2
--     2013/06/21    leushe           include VXX tables
--
------------------------------------------------------------------------

IS
   l_del_count      INTEGER DEFAULT 0;
   l_rows_deleted   INTEGER DEFAULT 0;
   l_id             NUMBER DEFAULT -99;
   l_cnt            integer;

   l_stmtTemplate   VARCHAR2 (1000);
   at_end           SMALLINT DEFAULT 0;
   stmt_end         SMALLINT DEFAULT 0;
   l_delStmt        VARCHAR2 (1000);
   v_sqlstate       VARCHAR2 (5) DEFAULT '00000';     -- variable for SQLSTATE
   v_sqlcode        INTEGER DEFAULT 0;                 -- variable for SQLCODE
   v_delete_count   INTEGER;                 -- variable for deleted row count

   TYPE refcur IS REF CURSOR;

   c_stmt           refcur;
BEGIN
   /* Initialize variables */
   --VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
   --o_tabname:=i_tabname;

   l_stmtTemplate :=
         'select cmx_tran_id from '
      || RTRIM (i_tabname)
      || ' where ( rqo_proc_utc_datetime <= :1 and rownum <= 100 )   ';


   --
   -- cursor(s) to delete the rows returned from the driving cursor
   --
   l_delStmt :=
      'DELETE FROM ' || RTRIM (i_tabname) || ' WHERE cmx_tran_id = :1 ';


   --
   -- sanity check that only the transaction tables are passed in
   --
   select count(*) into l_cnt from FCM_TRANSACTION_DEFINITION
                where sor_Table_name = i_tabname
                    or sor_Table_name = substr(i_tabname,1,8)
                    ;
    if (l_cnt = 0) then                    
                 v_sqlcode := -1;
                 v_sqlstate := '-1';
                 v_delete_count := 0;

   ELSE
      v_sqlstate := '00000';
      at_end := 0;
      l_rows_deleted := 0;
      l_del_count := 0;
      stmt_end := 0;

      WHILE (v_sqlstate = '00000' AND stmt_end = 0)
      LOOP
         OPEN c_stmt FOR l_stmtTemplate USING i_start_date;


         FETCH c_stmt INTO l_id;

         IF c_stmt%NOTFOUND
         THEN
            stmt_end := 2;
            at_end := 1;
         END IF;

         WHILE (v_sqlstate = '00000' AND at_end = 0)
         LOOP
            IF (v_sqlstate = '00000' AND at_end = 0)
            THEN
               EXECUTE IMMEDIATE l_delStmt USING l_id;

               l_del_count := l_del_count + 1;

               IF (l_del_count = i_commit_point)
               THEN
                  COMMIT;
                  l_rows_deleted := l_rows_deleted + l_del_count;
                  l_del_count := 0;
               END IF;
            END IF;

            FETCH c_stmt INTO l_id;

            IF c_stmt%NOTFOUND
            THEN
               at_end := 1;
            END IF;
         END LOOP;

         at_end := 0;
         CLOSE c_stmt;
      END LOOP;
         IF (l_del_count > 0)
         THEN
            COMMIT;
            l_rows_deleted := l_rows_deleted + l_del_count;
         END IF;




         v_delete_count := l_rows_deleted;
   END IF;

   INSERT INTO fmx_job_run_msg (job_run_msg_id,
                                job_id,
                                job_run_id,
                                msg_type,
                                MESSAGE)
        VALUES (
                  job_run_msg_seq.NEXTVAL,
                  i_job_id,
                  i_job_run_id,
                  'PROC_OUT',
                     i_tabname
                  || ','
                  || v_delete_count
                  || ','
                  || v_sqlstate
                  || ','
                  || v_sqlcode);
END DELETE_XACTIONS;
/

Show errors;
