
  CREATE OR REPLACE PROCEDURE GET_ALERT_FROM_QUEUE (
    i_user_id          IN NUMBER,
    I_USER_NAME        IN VARCHAR2,
    i_queue_id         IN NUMBER,
    i_queue_sort_field IN VARCHAR2,
    i_queue_sort_order IN VARCHAR2,
    o_alert_id OUT NUMBER )
  -----------------------------------------------------------------------------
  -- File:  GET_ALERT_FROM_QUE
  --
  --
  -- Description:
  --
  --
  -- Called By:
  --     qry_alert_service
  --
  -- Calls:
  --
  --
  -- Notes:
  --
  --
  -- Usage:
  --
  --
  -- History:    
  --     2012/07/23    leushe				 Ported from DB2
  --     2013/03/12    leushe      Change support to alert/entity values of 100 bytes
  --     2014/02/05    leushe      Changed the OR to an AND in the Alert Suspension check
  --  							       /* If the insert fails, then the update should fail also */
  --                                   IF(L_ROW_COUNT = 0 AND L_CONTACT_VALUE IS NOT NULL) THEN
  ------------------------------------------------------------------------
is
  l_alert_id number DEFAULT -1;
  l_row_count         INT DEFAULT 0;
  SQLSTATE CHAR(5);
  v_sqlstate          VARCHAR2(5) DEFAULT '00000';
  --SQLSTATE            CHAR(5) DEFAULT '00000';

  l_contact_value          VARCHAR2(100);
  l_contact_Type      varchar2(1);
  l_ALERT_VERSION_NBR INTEGER;
  l_ALERT_VALUE       VARCHAR2(100);
  l_ALERT_TYPE        VARCHAR2(1);
  l_TENANT_MULTI_ORG_ID      INTEGER;
  l_MULTI_ORG_ID      INTEGER;
  l_GMT_OFFSET        NUMBER(3,1);
  l_ALERT_SERVICE_CNT INTEGER;
  l_ALERT_ORIGIN      VARCHAR2(20);
  l_CREATE_USER       VARCHAR2(60);
  l_CREATE_TIMESTAMP TIMESTAMP;

   l_sort              VARCHAR2(4) DEFAULT 'desc';
   l_baseSql           VARCHAR2(2500);
   l_suspend           VARCHAR2(5);
     --  c_stmt CURSOR for l_stmt;
       TYPE refcur IS REF CURSOR;
      c_stmt refcur;
--
BEGIN


  --Check to see if suspended alerts is on
  --
  --SELECT COALESCE(SUBSTRING(LOWER(PROPERTY_VALUE),1,5),'false') --as s INTO l_suspend
  select count(*)  into l_row_count FROM FCM_PROPERTY    WHERE PROPERTY_NAME = 'analyst.alert.suspend.enabled';
  if (l_row_count > 0) then
  	SELECT COALESCE(rtrim(LOWER(PROPERTY_VALUE)),'false') INTO l_suspend   FROM FCM_PROPERTY    WHERE PROPERTY_NAME = 'analyst.alert.suspend.enabled';
  else
     L_SUSPEND     := 'false';
  END IF;
  l_row_count:=0;
  IF ( i_queue_sort_order = 'ASC' ) THEN
     l_sort            := ' ';
  END IF;
--
   l_baseSql :=  'select  * from (select alert_id,
				          contact_value,
					       contact_type,
					       ALERT_VERSION_NBR,ALERT_VALUE,
                            ALERT_TYPE,
							TENANT_MULTI_ORG_ID,
                            MULTI_ORG_ID,
                            GMT_OFFSET,
                            ALERT_SERVICE_CNT,
                            ALERT_ORIGIN,
                            aa.CREATE_USER,
                            aa.CREATE_TIMESTAMP
                            from fcm_alert aa,
                             (SELECT Q2.queue_id queue_id,
                                      Q2.queue_call_superseded_ind queue_call_superseded_ind,
                                      Q2.queue_call_lower_bound queue_call_lower_bound,
                                      Q2.queue_call_upper_bound queue_call_upper_bound
                                      FROM fcm_queue Q2
                                      where q2.queue_id = :queue_id
                                      ORDER BY Q2.QUEUE_ID) Q1
                            where aa.queue_id = :queue_id2
                            and CHECK_MO_AT_ACCESS( :userId , aa.multi_org_id, aa.alert_type) = 1
                            and aa.queue_id = Q1.queue_id
                            and aa.alert_status = ''ACTIVE''
                            and aa.next_service_timestamp < SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)
                            and (Q1.queue_call_superseded_ind = ''1''
                            or curr_cust_hour(aa.gmt_offset) between Q1.queue_call_lower_bound and Q1.queue_call_upper_bound)
                            and NOT exists (select 1
                                      from fcm_alert_suspend
                                      where  contact_value = aa.contact_value 
									    and contact_type = aa.contact_type
									    and TENANT_MULTI_ORG_ID = aa.TENANT_MULTI_ORG_ID
                                        and suspend_user <> :username
                                      )
                            order by aa.'||i_queue_sort_field||' '||l_sort||' ) where rownum < 10 ';


OPEN c_stmt FOR l_baseSql USING i_queue_id,i_queue_id,i_user_id,i_user_name;
  FETCH   c_stmt  INTO l_alert_id,
                            l_contact_value,l_contact_type,
                            l_ALERT_VERSION_NBR ,
                            l_ALERT_VALUE ,
                            l_ALERT_TYPE ,
                            l_TENANT_MULTI_ORG_ID,
                            l_MULTI_ORG_ID,
                            l_GMT_OFFSET ,
                            l_ALERT_SERVICE_CNT ,
                            l_ALERT_ORIGIN ,
                            l_CREATE_USER ,
                            l_CREATE_TIMESTAMP;
   IF (c_stmt%FOUND) THEN
      v_sqlstate   := '00000';
   ELSE
      v_sqlstate   := '99999';
   END IF;



  WHILE(v_sqlstate = '00000') LOOP
  -- Security check for the user should have been done in the l_baseSql statement, so no need
  -- to check security on this update
      IF (l_suspend = 'true') THEN
        INSERT INTO fcm_alert_suspend
          (contact_value,contact_Type,TENANT_MULTI_ORG_ID,alert_id,suspend_user,suspend_timestamp)
	        SELECT l_contact_value,l_contact_Type,TENANT_MULTI_ORG_ID,
			          alert_id,
			          i_user_name,
			          SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)
	        FROM fcm_alert 
	        WHERE NOT EXISTS
	          (SELECT 1
		          FROM fcm_alert_suspend
		          WHERE contact_value = l_contact_value 
						and contact_type = l_contact_Type
						and TENANT_multi_org_id = l_TENANT_MULTI_ORG_ID
		            /* commented out for race condition*/
		            --         and suspend_user <> i_user_name
		            --         and alert_id <> l_alert_id
		          )
	        AND (next_service_timestamp) < SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)
	        AND FCM_ALERT.CONTACT_VALUE IS NOT NULL
			AND alert_id                 = l_alert_id
	        AND alert_status             ='ACTIVE';

         l_row_count := SQL%ROWCOUNT;
        /* If the insert fails, then the update should fail also */
        IF(L_ROW_COUNT = 0 AND L_CONTACT_VALUE IS NOT NULL) THEN
          UPDATE FCM_ALERT_SUSPEND
          SET ALERT_CHECKOUT_CNT      = ALERT_CHECKOUT_CNT+1
          WHERE contact_value = l_contact_value 
				and contact_type = l_contact_Type
				and TENANT_multi_org_id = l_TENANT_MULTI_ORG_ID 
          		AND SUSPEND_USER            = I_USER_NAME;

           l_row_count := SQL%ROWCOUNT;
        END IF;

        IF(l_row_count = 1 OR L_CONTACT_VALUE IS  NULL) THEN
          --
          UPDATE fcm_alert
          SET lstupdt_user         = i_user_name,
            alert_status           = 'CHECKOUT',
            alert_service_cnt      = alert_service_cnt + 1,
            checkout_user          = i_user_name,
            checkout_timestamp     = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
            user_service_timestamp = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
            lstupdt_timestamp      = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
            lstupdt_nbr            = lstupdt_nbr       + 1,
            pause_timestamp        = NULL,
            pause_elapsed_time     = 0
          WHERE alert_status       = 'ACTIVE'
          AND alert_id             = l_alert_id ;

           l_row_count := SQL%ROWCOUNT;

          IF(l_row_count              = 1) THEN
             v_sqlstate            := '00001';
          ELSE
             l_alert_id := -1;
            FETCH
             c_stmt
            INTO l_alert_id,
              l_contact_value,l_contact_Type,
              l_ALERT_VERSION_NBR ,
              l_ALERT_VALUE ,
              l_ALERT_TYPE ,
              l_TENANT_MULTI_ORG_ID,
              l_MULTI_ORG_ID,
              l_GMT_OFFSET ,
              l_ALERT_SERVICE_CNT ,
              l_ALERT_ORIGIN ,
              l_CREATE_USER ,
              l_CREATE_TIMESTAMP;
               IF (c_stmt%FOUND) THEN
                  v_sqlstate   := '00000';
               ELSE
                  v_sqlstate   := '99999';
               END IF;
          END IF;
        ELSE
           l_alert_id := -1;
          FETCH  c_stmt
          INTO l_alert_id,
	            l_contact_value,l_contact_Type,
	            l_ALERT_VERSION_NBR ,
	            l_ALERT_VALUE ,
	            l_ALERT_TYPE ,
	            l_TENANT_MULTI_ORG_ID,
	            l_MULTI_ORG_ID,
	            l_GMT_OFFSET ,
	            l_ALERT_SERVICE_CNT ,
	            l_ALERT_ORIGIN ,
	            l_CREATE_USER ,
	            l_CREATE_TIMESTAMP;
           IF (c_stmt%FOUND) THEN
              v_sqlstate   := '00000';
           ELSE
              v_sqlstate   := '99999';
           END IF;
        END IF;
  ELSE
        UPDATE fcm_alert
        SET lstupdt_user         = i_user_name,
          alert_status           = 'CHECKOUT',
          alert_service_cnt      = alert_service_cnt + 1,
          checkout_user          = i_user_name,
          checkout_timestamp     = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
          user_service_timestamp = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
          lstupdt_timestamp      = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
          lstupdt_nbr            = lstupdt_nbr       + 1,
          pause_timestamp        = NULL,
          pause_elapsed_time     = 0
        WHERE alert_status       = 'ACTIVE'
        AND alert_id             = l_alert_id
          /*with UR*/
          ;
         l_row_count := SQL%ROWCOUNT;

        IF(l_row_count              = 1) THEN
           v_sqlstate            := '00001';
        ELSE
           l_alert_id := -1;
          FETCH  c_stmt
          INTO l_alert_id,
                l_contact_value,l_contact_Type,
                l_ALERT_VERSION_NBR ,
                l_ALERT_VALUE ,
                l_ALERT_TYPE ,
                l_TENANT_MULTI_ORG_ID,
                l_MULTI_ORG_ID,
                l_GMT_OFFSET ,
                l_ALERT_SERVICE_CNT ,
                l_ALERT_ORIGIN ,
                l_CREATE_USER ,
                l_CREATE_TIMESTAMP;
             IF (c_stmt%FOUND) THEN
                v_sqlstate   := '00000';
             ELSE
                v_sqlstate   := '99999';
             END IF;
        END IF;
      END IF;
END LOOP;
CLOSE c_stmt;
/*
regardless of success ( alert_id > 0) or failure ( alert_id = -1 )
commit to release all resources  - necessary because of the select for update
*/
COMMIT;
 o_alert_id := l_alert_id;
END GET_ALERT_FROM_QUEUE;

/
show errors;


